﻿
Module Module_Buffer

    Friend DataArray1 As DataArray = New DataArray(100000)

End Module


Public Class DataArray

    Friend Structure DataSample
        Dim x As Single
        Dim y As Single
        Dim z As Single
        Dim t As Long
        Sub New(ByVal _x As Single, _
                ByVal _y As Single, _
                ByVal _z As Single, _
                ByVal _t As Long)
            x = _x
            y = _y
            z = _z
            t = _t
        End Sub
    End Structure

    Private m_ArrayMaxLength As Int32
    Private m_WriteIndex As Int32
    Private m_DataArray() As DataSample
    Private m_NumEntries As Int32

    Friend Sub New(ByVal ArrayMaxLength As Int32)
        m_ArrayMaxLength = ArrayMaxLength
        ReDim m_DataArray(m_ArrayMaxLength - 1)
        Flush()
    End Sub

    Friend Sub Flush()
        m_WriteIndex = 0
        m_NumEntries = 0
    End Sub

    Friend Function GetWriteIndex() As Int32
        Return m_WriteIndex
    End Function

    Friend Sub SetWriteIndex(ByVal WriteIndex As Int32)
        If WriteIndex < 0 Then WriteIndex = 0
        If WriteIndex > m_ArrayMaxLength - 1 Then WriteIndex = m_ArrayMaxLength - 1
        m_WriteIndex = WriteIndex
    End Sub

    Friend Sub SetNumEntries(ByVal n As Int32)
        If n < 0 Then n = 0
        If n > m_ArrayMaxLength - 1 Then n = m_ArrayMaxLength - 1
        m_NumEntries = n
    End Sub

    Friend Function GetNumEntries() As Int32
        Return m_NumEntries
    End Function

    Friend Function GetBuffer() As DataSample()
        Return m_DataArray
    End Function

    Friend Sub AddElement(ByVal x As Single, _
                          ByVal y As Single, _
                          ByVal z As Single, _
                          ByVal t As Long)
        Dim ds As DataSample
        ds = New DataSample(x, y, z, t)
        m_DataArray(m_WriteIndex) = ds
        m_WriteIndex += 1
        If m_WriteIndex > m_ArrayMaxLength - 1 Then m_WriteIndex = m_ArrayMaxLength - 1
        m_NumEntries += 1
        If m_NumEntries > m_ArrayMaxLength Then m_NumEntries = m_ArrayMaxLength
    End Sub

    Friend Sub FillWithInvalidData()
        Dim ds As DataSample = New DataSample(-99999, -99999, -99999, -99999)
        For i As Int32 = 0 To m_ArrayMaxLength - 1
            m_DataArray(i) = ds
        Next
    End Sub

    Friend Function GetElement(ByVal Index As Int32) As DataSample
        If Index < 0 Then Index = 0
        If Index > m_ArrayMaxLength - 1 Then Index = m_ArrayMaxLength - 1
        Return m_DataArray(Index)
    End Function


    ' =============================================================
    '  SAVE LOAD BUFFER
    ' =============================================================

    Private LogFilesFolder As String = Application.StartupPath + "\Buffers\"
    Private LogFileName As String

    Friend Sub SaveLastBuffer()
        IO.Directory.CreateDirectory(LogFilesFolder)
        Dim s As String = LogFilesFolder + "LastBuffer.csv"
        SaveBufferTo(s)
    End Sub

    Friend Sub SaveBuffer_Dialog()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = LogFilesFolder
        sfd.DefaultExt = ".csv"
        sfd.AddExtension = True
        sfd.FileName = "Buffer_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        sfd.Filter = "Buffer file (*.csv)|*csv"
        IO.Directory.CreateDirectory(sfd.InitialDirectory)
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveBufferTo(sfd.FileName)
        End If
    End Sub

    Friend Sub SaveBufferTo(ByVal filename As String)
        Using sw As IO.StreamWriter = New IO.StreamWriter(filename)
            For i As Int32 = 0 To m_NumEntries - 1
                With m_DataArray(i)
                    sw.WriteLine(.x.ToString("000.000", GCI) + ", " + _
                                 .y.ToString("000.000", GCI) + ", " + _
                                 .z.ToString("000.000", GCI) + ", " + _
                                 .t.ToString())
                End With
            Next
        End Using
    End Sub

    Friend Sub LoadBuffer_Dialog()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = LogFilesFolder
        IO.Directory.CreateDirectory(ofd.InitialDirectory)
        ofd.Filter = "All files (*.*)|*.*" ' "Buffer file (*.csv)|*.csv"
        ofd.DefaultExt = ".csv"
        ofd.Multiselect = True
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            LogFileName = ofd.FileName
            LoadBufferFrom(LogFileName)
        End If
    End Sub

    Friend Sub LoadLastBuffer()
        Dim s As String = LogFilesFolder + "LastBuffer.csv"
        LoadBufferFrom(s)
    End Sub

    Friend Sub LoadBufferFrom(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            ' -------------------------------------------------------
            Flush()
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(filename)
            Dim i As Int32
            ' -------------------------------------------------------
            For i = 0 To m_DataArray.Length - 1
                If f.EndOfStream Then Exit For
                Dim p() As String = f.ReadLine.Split(","c)
                With m_DataArray(i)
                    If p.Length = 4 Then
                        .x = CSng(Val(p(0)))
                        .y = CSng(Val(p(1)))
                        .z = CSng(Val(p(2)))
                        .t = CLng(Val(p(3)))
                    End If
                End With
            Next
            f.Close()
            m_NumEntries = i - 1
            m_WriteIndex = i
        End If
    End Sub

End Class
